/**
 * DESCRIPTION: LOG API REQUEST AND RESPONSE IN THE DB TABLE
 * CREATED BY: RAVIRAJ
 * CREATED DATE: 25/03/2021
 * CREATED BY: VIKAS
 * CREATED DATE: 11/05/2021
 */
import { Request, Response } from "express";
import { APIUsers } from "../dataModels/Kinara_Engr_Api_Users";
import { APIToken } from "../dataModels/Kinara_Engr_Api_Tokens";
import { APIHistoryTable } from "../dataModels/kinara_engr_api_hist_table";

/**
 * @desc This async function return user_name base on token.
 *
 * @param  {string} token Api token.
 * @return {JSON} user_name from database.
 */
async function get_user_id(token: string) {
  const response = await APIToken.findOne({
    attributes: ["user_Id"],
    where: { token: token },
    raw: true,
  }).then(async (result: any) => {
    return await APIUsers.findOne({
      attributes: ["user_name"],
      where: { ID: result.user_Id },
      raw: true,
    });
  });
  console.log(response);
  return response!;
}

/**
 * In this we are save api calling in history tables.
 * Note: don't send server response.
 * @param  {Request} req            APi request
 * @param  {Response} res           APi response json, this should be json, created for api response.
 * @param  {String} apiErrorLabel   APi error reason, if success then NA,
 *                                  other reason eg. 500 --> internal error.
 * @param  {string} apiStatus       Api status  it can be success or fail.
 * @param  {string} logicStatus     Api logic status  it can be success or fail.
 * @param  {Number} apiStatusCode   Api status code, it can be 500, 200, etc.
 *
 * @return {Boolean} true if successfully inserted otherwise false.
 */
export const add_api_call = async (
  req: Request,
  res: Response,
  apiErrorLabel: string,
  apiStatus: string,
  logicStatus: string,
  apiStatusCode: number
) => {
  try {
    // Fetch the token from request header
    let token = <string>req.headers["authorization"]?.split(" ")[1];

    // Get user-id
    await get_user_id(token).then(async (result: any) => {
      // Fetch user_name else "NA"
      let apiUserId = result.user_name || "NA";
      console.log(apiUserId);

      // Get the original URL
      let apiRoute = req.originalUrl;

      // Considering last value of a string,
      // Ex. /api/colending --> colending
      let apiName = apiRoute.split("/")[1];

      // Inserting api details in api_hist table
      let response = await APIHistoryTable.create({
        application_id: "N/A",
        user_id: apiUserId,
        api_route: apiRoute,
        api_name: apiName,
        api_error_label: apiErrorLabel,
        status_code: apiStatusCode,
        api_status: apiStatus,
        logic_status: logicStatus,
        // Converting JSON (req and res) to string and replacing `"` to `'` while inserting value.
        api_request: JSON.stringify(req.body).replace(/"/g, `'`),
        api_response: JSON.stringify(res).replace(/"/g, `'`),
        server_type: <string>process.env.SERVER_TYPE, // Fetch server type from .env file
        // Current Datetime with India local time zone
        api_request_on: new Date().toLocaleString("en-US", {
          timeZone: "Asia/Calcutta",
          hour12: false,
        }),
      });

      // Capture the Log
      console.log(response instanceof APIToken);
      return response instanceof APIToken;
    });
  } catch (ex) {
    console.log(`API History Exception Catch - ${ex}`);
  }
};
